module derived_types

  use contour_531, only: polyline
  use shapelib, only: shpfileobject
  
  implicit none

  type, extends(polyline):: ssh_contour
     ! Points of polyline are (longitude, latitude) in rad. Polyline
     ! should be closed, except if it is null.
     real ssh ! in m
     real area ! in m2
  end type ssh_contour

  type eddy
     real coord_extr(2) ! longitude and latitude of the extremum, in rad
     real ssh_extr ! extremum value of ssh, in m
     logical cyclone
     type(ssh_contour) out_cont ! outermost contour
     type(ssh_contour) speed_cont ! contour with maximum average azimuthal speed

     real max_speed
     ! average of azimuthal speed on out_cont or speed_cont, in m s-1

     logical valid ! valid out_cont found: not a null contour, and
                   ! with sufficient area

     integer:: delta_in = huge(0)
     ! Minimum difference in time subscript where there is a direct
     ! predecessor. Huge means there is no direct predecessor.
     
     integer:: delta_out = huge(0)
     ! Minimum difference in time subscript where there is a direct
     ! successor. Huge means there is no direct succcessor.

     integer radius4
     ! ind_extr + radius4 - 1 in all four directions is inside outermost
     ! contour, ind_extr + radius4 is outside.
  end type eddy

  type snapshot
     type(eddy), allocatable:: list_vis(:) ! (number_extr)
     ! Visible "eddies" at a given date. These include "eddies"
     ! without an outermost contour, that is only the extremum is
     ! visible. The subscript value in list_vis is the "identification
     ! number" of the extremum.
     
     integer number_extr ! number of visible extrema

     integer, allocatable:: extr_map(:, :)
     ! (1 - copy:nlon + copy, nlat) if the grid is periodic in
     ! longitude, else (nlon, nlat). copy can be max_radius(1) or
     ! dist_lim, depending on the program. At a point of extremum SSH:
     ! identification number or this extremum. 0 at other points.

     integer, allocatable:: ind_extr(:, :) ! (2, number_extr)
     ! List of coordinates of ssh extrema in the global grid, in subscript
     ! space. The position in this list is the identifying number of
     ! the extremum.
  end type snapshot

  real, parameter:: missing_ssh = 1e4 ! flag for undefined SSH
  real, parameter:: missing_speed = 1e4 ! flag for missing max speed

  type shpc ! shapefile collection
     TYPE(shpfileobject) extremum ! shapefile extremum
     TYPE(shpfileobject) outermost ! shapefile outermost_contour
     TYPE(shpfileobject) max_speed ! shapefile max_speed_contour
     logical cyclone
     
     ! Field identifiers:
     integer extr_ssh, extr_date, extr_eddy_index, extr_valid, extr_speed
     integer out_r_eq_area, out_ssh, out_date, out_eddy_index, out_radius4
     integer max_speed_r_eq_area, max_speed_ssh, max_speed_date, &
          max_speed_eddy_index
  end type Shpc

  private shpfileobject

contains

  pure type(ssh_contour) function null_ssh_contour()

    use contour_531, only: null_polyline

    !--------------------------------------------------------------

    null_ssh_contour%polyline = null_polyline()
    null_ssh_contour%ssh = missing_ssh
    null_ssh_contour%area = - 1e6

  end function null_ssh_contour

end module derived_types
